# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2007-TODAY ASPerience SARL (<https://www.asperience.fr>).
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import osv
import time
from tools.translate import _
import logging

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def run_autopay(self, cr, uid, context={}):
        logger = logging.getLogger("asperience_account_invoice_autopay_if_null")
        period_obj = self.pool.get('account.period')
        move_obj = self.pool.get('account.move')
        move_line_obj = self.pool.get('account.move.line')
        invoice_ids = self.search(cr,uid,['|',('residual','=',False),('residual','=',0.0),('state','=','open')])
        
        for invoice in self.browse(cr,uid,invoice_ids):
            logger.info("Autopay invoice %s analysis" % (invoice.number,))
            for move_line in invoice.move_id.line_id:
                if not move_line.reconcile_id and move_line.account_id.id == invoice.account_id.id:
                    if move_line.debit == 0.0 and move_line.credit == 0.0:
                        logger.info("Start autopay account move line %s for invoice %s" % (move_line.id,invoice.number))
                        iml = []
                        dataline = {
                            'type': 'dest',
                            'name': _('Autopay null invoice move lines'),
                            'price': 0.0,
                            'account_id': move_line.account_id.id,
                            'date_maturity': False,
                            'amount_currency': False,
                            'currency_id': False,
                            'ref': False
                        } 
                        iml.append(dataline)
                        date = time.strftime('%Y-%m-%d')
                        context.update({'date': date})
                        journal_id = move_line.journal_id.id
                        partner_id = move_line.partner_id.id
                
                        lines = map(lambda x:self.line_get_convert(cr, uid, x, partner_id, date, {}),iml)
                        
                        move = {
                            'ref': dataline['name'],
                            'journal_id': journal_id,
                            'date': date,
                        }
                        context.update(company_id=move_line.company_id.id,
                               account_period_prefer_normal=True)
                        period_ids = period_obj.find(cr, uid, date, context=context)
                        period_id = period_ids and period_ids[0] or False
                        line_ids = []
                        if period_id:
                            move['period_id'] = period_id
                            for line in lines:
                                line['period_id'] = period_id
                                line['journal_id'] = journal_id
                                move_line_id = move_line_obj.create(cr, uid, line, context=context)
                                line_ids.append(move_line_id)
                            
                            move['line_id'] = [(6,0,line_ids)]
                            move_obj.create(cr, uid, move, context=context)
                            line_ids.append(move_line.id)
                            move_line_obj.reconcile(cr, uid, line_ids, 'manual', context=context)
                        
                        cr.commit()
                        logger.info("Account move line %s paid for invoice %s" % (move_line.id,invoice.number))
                    else:
                        logger.info("Line %s not null : %s / %s" % (move_line.id,move_line.debit,move_line.credit))
                else:
                    logger.info("Line %s not autopayable" % (move_line.id,))
    
account_invoice()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: